backbone-view-states.js
====================
[![Build Status](https://travis-ci.org/bsabadach/backbone-view-states.png?branch=master)](https://travis-ci.org/bsabadach/backbone-view-states)


#Goal
**backbone-view-states.js** is an extension on top Backbone.js that proposes an UI component creation model based on declarations of view states.

The library is composed of an enhanced Backbone View class that manage lifecycle of declared states, and listen to the eventual transitions between these states.

## Requirements

+ [grunt-cli](http://gruntjs.com/getting-started): run `npm install -g grunt-cli` if needed.
+ backbone-view-states has tight dependency to Backbone 0.9.


##API

The extension adds to the `Backbone` object:

+ the `StatesClientView` class: a statefull extendable View with state life cycle management features.
+ the `makeAsStateClient` function: to add state management suppor to your own Backbone view class.



##Usage

### basic: JavaScript, CSS, and HTML states declarations.

create an HTML node on which you want to manage states by

+ declaring states names on the root node with `data-states` attribute
+ declaring children which appearance is supposed to change on each state by using `data-states-inc` and `data-states-exc` attributes
  `data-states-inc` means the element is included in the declared states, aa`data-states-out` means the element is excluded from
   the declared states.

        <div id="state-container" data-states="A,B,C">
            <div class="child-one" data-states-inc="A">
                <span>included in A</span>
            </div>
            <div class="child-two" data-states-inc="A,B">
                <span>included in A,B</span>
            </div>
            <div class="child-three" data-states-exc="A">
                <span>excluded from A</span>
            </div>
        </div>


+ adding necessary CSS classes.  For instance here, the div with child-two style class is included in states A and B
Therefore in CSS, there has to be at least three style classes: `child-two` , `child-two-A` , `child-two-B` .
+ Instantiate a state client view with the root node id:

    `var myView=new BackBone.StatesClientView({el;$(selector),states:{options});`

+ wait for the DOM to be ready to render the view.



**The options hash**:

All the options non required and have a default value.

**`includeRoot`**
default:  false, include the root node in all states if set to true.

**`keepLayout`**
default: false; apply a different display strategy when showing, hiding elements: true will use the `visibility` property,
while false will use the `display` CSS property

**`initialState`**
 default: the first state declared on the root node.

**`deepFetch`**
 default: false; search only for direct children with data-states-* attributes if false.

**`reverseTransitions`**
 default: false; if the browser supports CSS transitions and a transition is declared in the style class of an element for a state, the transition will play in reverse when the component will leave this state 

**`silent`**
 default: false; if false, the view triggers states life cycle events.
`Backbone.States.ENTER,  Backbone.States.EXIT, Backbone.States.EXITED, Backbone.States.CHANGED`.


### DOM unobtrusive

It is possible to add states to a node by declaring them programmatically in the constructor options, without using `data-states-*` attributes

Here is the syntax for the previous example node

        var myView=new Backbone.StatesClientView({
                        el:$(selector),
                        states:{
                            names:"A,B,C",
                            option1:.. ,
                            option2:.. ,
                            targets:{
                                ".child-one":{inc:"A"},
                                ".child-two":{inc:"A,B"},
                                ".child-three":{exc:"A"}
                            }
                        }
                    });
                                            
Or it could directly be added in the class creation as defaults:

        var MyViewClass=Backbone.StatesClientView.extend({
                            statesDefaults:{
                                names:"A,B,C",
                                option1:.. ,
                                option2:.. ,
                                targets:{
                                    ".child-one":{inc:"A"},
                                    ".child-two":{inc:"A,B"},
                                    ".child-three":{exc:"A"}
                                }
                            }
                        });

##Examples

you can find examples on these two posts on washmatique.fr , [post 1](http://washmatique.fr/blog/2012/11/23/create-multi-states-ui-components-for-backbone-js.html), and  [post 2](http://washmatique.fr/blog/2013/09/10/create-accordion-component-with-backbone-view-states.js.html) 
